/**
  ******************************************************************************
  * @file    function.c
  * @brief   This file contains the utility functions used for the demo
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "functions.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define RAM1_START            0x000
#define RAM1_END              0x7FF


// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
__IO u32 GTimeStamp = 0x00000000;
u16 TimeOutStartTime;
u8 EEPBuffer[16];
u8 GUseEeprom;

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  ******************************************************************************
  * @brief Initialize the RAM
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void InitRAM1(void)
{
  unsigned char *pRAM;
  // To initialize the RAM1 
  for (pRAM = (unsigned char*)RAM1_START; pRAM <= (unsigned char*)RAM1_END; pRAM++)
  {
    *pRAM = 0;
  }
}

/**
  ******************************************************************************
  * @brief Initialize the SPI for LCD com
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_SPISimplex(void)
{

  // SPI clock on
  CLK->PCKENR1 |= 0x10;

  // init SPI
  SPI_DeInit(SPI1);
  SPI_Init(SPI1, SPI_FirstBit_MSB, SPI_BaudRatePrescaler_128, SPI_Mode_Master,
           SPI_CPOL_High, SPI_CPHA_2Edge, SPI_Direction_1Line_Tx, SPI_NSS_Soft, 0x07);
  SPI_Cmd(SPI1, ENABLE);

  // For the LCD display 
  WaitDelay(200);

  // Remap the SPI1 pins 
  SYSCFG->RMPCR1 |= 0x80;
}

/**
  ******************************************************************************
  * @brief Initialize the TIM1 (PWM for buzzer: TIM1_CC4 )
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_TIM1(void)
{
  // Enable the TIMER1 clock
  CLK->PCKENR2 |= 0x02;

  // configure the GPIO (PE1) used for managing the BEEPER
  GPIOE->ODR |= 0x02;
  GPIOE->DDR |= 0x02;
  GPIOE->CR1 |= 0x02;

  TIM1_PrescalerConfig((u16)3, TIM1_PSCReloadMode_Immediate);

  TIM1_OC2Init( TIM1_OCMode_PWM1,  TIM1_OutputState_Disable, TIM1_OutputNState_Enable, 0xFFFF,  TIM1_OCPolarity_Low, TIM1_OCNPolarity_High, TIM1_OCIdleState_Reset, TIM1_OCNIdleState_Reset);

  TIM1_CtrlPWMOutputs(ENABLE);

  TIM1_Cmd(ENABLE);

}

/**
  ******************************************************************************
  * @brief General Purpose 5ms Time base
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_TIM3(void)
{

  /* TimerTick = 5 ms
     Warning: fcpu must be equal to 16MHz
     fck_cnt = fck_psc/presc = fcpu/128 = 125 kHz --> 1 tick every 8 s
     ==> 5 ms / 8 s = 625 ticks
   */

  // Enable the TIMER3 clock
  CLK->PCKENR1 |= 0x02;

  TIM3_TimeBaseInit( TIM3_Prescaler_128, TIM3_CounterMode_Up, 625);

  TIM3_UpdateRequestConfig(TIM3_UpdateSource_Global);

  TIM3_ITConfig(TIM3_IT_Update, ENABLE);
  TIM3_Cmd(ENABLE);

}

/**
  ******************************************************************************
  * @brief General Purpose I/O init for LED
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_GPIO(void)
{
	// Set all GPIOs to Output Push-Pull Low. Used GPIOs are configured by the application.	
  GPIOA->DDR |= 0xFE;
  GPIOA->CR1 |= 0xFE;
	GPIOB->DDR |= 0xFF;
  GPIOB->CR1 |= 0xFF;
  GPIOC->DDR |= 0xFF;
  GPIOC->CR1 |= 0xFF;
  GPIOD->DDR |= 0xFF;
  GPIOD->CR1 |= 0xFF;
  GPIOE->DDR |= 0x62;
  GPIOE->CR1 |= 0x62;
  GPIOF->DDR |= 0x01;
  GPIOF->CR1 |= 0x01;
	
  // Init I/O in Output pseudo opendrain to drive the LED 
  LED2_PORT_DDR |= LED2_PIN_MASK;
  LED2_PORT_CR1 &= (~LED2_PIN_MASK);
  LED2_PORT_ODR &= (~LED2_PIN_MASK); /* LED2 is ON by default */
	
}

/**
  ******************************************************************************
  * @brief Configure the fCPU
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_Clock(void)
{

  // Select fCPU = 16MHz 
  CLK->CKDIVR = 0x00; // Main freq divided by 1 = Full Freq

  // For test purpose output Fcpu on MCO pin 
  //CLK_CCOConfig(CLK_OUTPUT_CPU);

}

/**
  ******************************************************************************
  * @brief Configure the low power mode
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_LowPower(void)
{

  // fCPU = 16MHz 
	// fLSI = 32kHz 
	CLK_PeripheralClockConfig(CLK_Peripheral_RTC, ENABLE);
  RTC_DeInit();
	CLK_RTCClockConfig(CLK_RTCCLKSource_LSI, CLK_RTCCLKDiv_1);
  RTC_WakeUpClockConfig(RTC_WakeUpClock_RTCCLK_Div2);
	RTC_SetWakeUpCounter(0x0010u);
	RTC_ITConfig(RTC_IT_WUT, ENABLE);
	RTC_WakeUpCmd(ENABLE);
  //CLK_PeripheralClockConfig(CLK_Peripheral_RTC, DISABLE);

}

/**
  ******************************************************************************
  * @brief Initialize the application
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void Init_Application(void)
{

  // RAM initialization 
  // InitRAM1(); 

  // Variables initialization 
  ButtonStatus = 0;

  // Clock initialization 
  Init_Clock();

  // I/O Port initialization 
  Init_GPIO();

  // TIMER3 initialization for generic timebase 
  Init_TIM3();

  // SPI initialization for LCD 
  Init_SPISimplex();

  // LCD initialization: Init_SPISimplex must be executed before 
	LCD_Init();
  //LCD_Backlight(ENABLE);

  // Menu initialization 
  MenuInit();

  // TIMER1 initialization for buzzer (PWM mode)  
  Init_TIM1();

  // Low Power WakeUp initialization 
  Init_LowPower();

  // Enable interrupts 
  enableInterrupts();

}

/**
  ******************************************************************************
  * @brief Get the Timer time stamp
  * @param[in] None
  * @return u32 The timer time stamp
  ******************************************************************************
  */
u32 GetTimeStamp(void)
{
  return GTimeStamp;
}

/**
  ******************************************************************************
  * @brief Initialize the start time for generating a time-out
  * @param[in] Delay before time-out generation
  * @return None
  ******************************************************************************
  */
void GetTimeOutStartTime(void)
{
  sim();
  TimeOutStartTime = (u16)( GetTimeStamp() );
  rim();
}

/**
  ******************************************************************************
  * @brief Generate a time out after Delay
  * @note Before calling this function, the function GetTimeOutStartTime()
  * must be called.
  * @param[in] Delay before time-out generation based on timer tick
  * @return None
  ******************************************************************************
  */
bool TimeOut(u16 Delay)
{
  u16 TimeStamp;
  u16 PeriodStamp;

  sim();
  TimeStamp = (unsigned int)(GetTimeStamp());
  rim();

  PeriodStamp = TimeStamp - TimeOutStartTime;

  if (PeriodStamp >= Delay)
  {
    return TRUE;
  }
  else
  {
    return FALSE;
  }
}

/**
  ******************************************************************************
  * @brief Delay routine based on timer
  * @param[in] Delay based on timer Tick
  * @return None
  ******************************************************************************
  */
void WaitDelay(u16 Delay)
{

  u16 DelayPeriodStart;
  u16 TimeStamp;
  u16 PeriodStamp;

  sim();
  DelayPeriodStart = (u16)(GetTimeStamp());
  rim();

  for (;;)
  {
    sim();
    TimeStamp = (u16)(GetTimeStamp());
    rim();
    PeriodStamp = TimeStamp - DelayPeriodStart;
    if (PeriodStamp >= Delay)
    {
      return;
    }
  }

}

/**
  ******************************************************************************
  * @brief Play a sound on buzzer
  * @param[in] buzzer frequency
  * @return None
  ******************************************************************************
  */
void PlayPWM(u16 freq)
{
  u16 dutycycle;

  if (freq != 0)
  {
    // Set frequency 
    TIM1->ARRH = (u8)(freq >> 8);
    TIM1->ARRL = (u8)(freq);
    // Set duty cycle = freq/2 = 50% 
    dutycycle = (u16)(freq >> 4);
    TIM1->CCR2H = (u8)(dutycycle >> 8);
    TIM1->CCR2L = (u8)(dutycycle);
    // Enable OCN2 
    TIM1->CCER1 |= 0x40;
  }
}

/**
  ******************************************************************************
  * @brief STOP buzzer PWM
  * @param[in] None
  * @return None
  ******************************************************************************
  */
void StopPWM(void)
{
  // Disable OCN2 
  TIM1->CCER1 &= (u8)(~0x40);
}

// Public functions ----------------------------------------------------------

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
