/**
  ******************************************************************************
  * @file    joystick_button.c
  * @brief   This file contains the functions for Joystick management
  * @author  STMicroelectronics - MCD Application Team
  * @version V1.2.0
  * @date    04-NOV-2010
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------

#include "joystick_button.h"
#include "stm8_tsl_api.h"
#include "main.h"
#include "functions.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------

#define JOYSTICK_DEBOUNCE_NB   3 /* Number of samples to validate a joystick event */
#define BUTTON_DEBOUNCE_NB     3 /* Number of samples to validate a button event */

// Following timings are based on TimerTick 
#define JOYSTICK_PERIOD        8 /* Defines the period to read the joystick */
#define BUTTON_PERIOD          8 /*  Defines the period to read the button */

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
u8 ButtonStatus;
//u8 ForceUpdateDisplay;

// Private function prototypes -----------------------------------------------
// Private functions ---------------------------------------------------------

/**
  ******************************************************************************
  * @brief Read the joystick state
  * @param[in] None
  * @return JOYSTICK_ID_T The new joystick state
  ******************************************************************************
  */
JOYSTICK_ID_T ReadJoystick(void)
{

  u16 TimeStamp;
  u16 PeriodStamp;
  u8 CurrentJoystickState;
  static u16 JoystickPeriodStart;
  static u8 PreviousJoystickState;
  static u8 JoystickDebounceNb;

  sim();
  TimeStamp = (u16)(GetTimeStamp());
  rim();

  PeriodStamp = TimeStamp - JoystickPeriodStart;

  if (PeriodStamp >= JOYSTICK_PERIOD)
  {
    sim();
    CurrentJoystickState = (u8)(GPIO_ReadInputData(GPIOE) & 0x99);
    rim();
		
    if (CurrentJoystickState != PreviousJoystickState)
    {
      PreviousJoystickState = CurrentJoystickState;
      JoystickDebounceNb = 0;
      SetBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE);
      //ForceUpdateDisplay = 1;
    }
    else
    {
      if (ValBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE))
      {

        JoystickDebounceNb++;

        if (JoystickDebounceNb >= (JOYSTICK_DEBOUNCE_NB - 1))
        {
          switch (CurrentJoystickState)
          {
            case 0x98:
#if defined(BOARD_STMT_8L_EV1)
							// Left Button 
              CurrentJoystickState = LEFT;
#elif defined(BOARD_STM8L151CxT_TS1)            
              // Down Button 
              CurrentJoystickState = DOWN;
#endif             
              break;
            case 0x91:
#if defined(BOARD_STMT_8L_EV1)
							// Up Button 
              CurrentJoystickState = UP;
#elif defined(BOARD_STM8L151CxT_TS1)            
              // Left Button 
              CurrentJoystickState = LEFT;
#endif              
              break;
            case 0x19:
              // Right button 
              CurrentJoystickState = RIGHT;
              break;
            case 0x89:
#if defined(BOARD_STMT_8L_EV1)
							// Down Button 
              CurrentJoystickState = DOWN;
#elif defined(BOARD_STM8L151CxT_TS1)            
              // Up Button 
              CurrentJoystickState = UP;
#endif
              break;
            default:
              // Release Button 
              CurrentJoystickState = J_RELEASED;
              break;
          }
          ClrBit(ButtonStatus, BUTTON_STATUS_JOYSTICK_DEBOUNCE_ENABLE);
          return((JOYSTICK_ID_T)CurrentJoystickState);
        }
      }
    }
  }
  return(J_NONE);
}

/**
  ******************************************************************************
  * @brief Read the Button state
  * @param[in] None
  * @return BUTTON_ID_T The new button state
  ******************************************************************************
  */
BUTTON_ID_T ReadButton(void)
{
  static u16 ReadButtonPeriodStart;
  u16 TimeStamp;
  u16 PeriodStamp;
  u8 CurrentButtonState;
  static u8 PreviousButtonState;
  static u8 ButtonDebounceNb;

  sim();
  TimeStamp = (unsigned int)( GetTimeStamp() );
  rim();

  PeriodStamp = TimeStamp - ReadButtonPeriodStart;

  if ( PeriodStamp >= BUTTON_PERIOD )
  {

    while ( PeriodStamp >= BUTTON_PERIOD )
    { /* Adjust TimeStamp in a Period range */
      PeriodStamp -= BUTTON_PERIOD;
      ReadButtonPeriodStart += BUTTON_PERIOD;
    }

    // Read PE2 
    CurrentButtonState = (u8)(GPIO_ReadInputData(GPIOE) & 0x04);

    if (CurrentButtonState != PreviousButtonState)
    {
      PreviousButtonState = CurrentButtonState;
      ButtonDebounceNb = 0;
      SetBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE);
    }
    else
    {
      if (ValBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE))
      {
        ButtonDebounceNb++;
        if (ButtonDebounceNb >= (BUTTON_DEBOUNCE_NB - 1))
        {
          switch (CurrentButtonState)
          {
            case 0x04:
              // Button released
              CurrentButtonState = B_RELEASED;
              break;
            default:
              // Button pressed
              CurrentButtonState = B_PRESSED;
              break;
          }
          ClrBit(ButtonStatus, BUTTON_STATUS_BUTTON_DEBOUNCE_ENABLE);
          return((BUTTON_ID_T )CurrentButtonState);
        }
      }
    }
  }
  return(B_NONE);
}

// Public functions ----------------------------------------------------------

 // (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE***
